@@ -28,6 +28,8 @@ import warnings
 # FULMAR parts
 import fulmar.fulmar_constants as fulmar_constants
 from fulmar.func import (
+    mission_identifier,
+    target_identifier,
     read_lc_from_file,
     normalize_lc,
     time_flux_err,
@@ -38,33 +40,12 @@ from fulmar.func import (
 from fulmar.utils import (
     FulmarWarning,
     print_version,
-    rjd_to_astropy_time
+    rjd_to_astropy_time,
+    warning_on_one_line
 )
 
-from fulmar.mission_dic_manager import read_json_dic
-
-# # FONCTIONS A CLASSIFIER ######################################################################################
-
-# utils?
-
-
-# class FulmarWarning(Warning):
-#     """ Class form warning to be displayed as
-#     "FulmarWarning"
-#     """
-
-#     pass
-
-
-# def warning_on_one_line(message, category, filename, lineno,
-#                         file=None, line=None):
-#     return ' %s:%s: %s: %s' % (filename, lineno, category.__name__, message)
-
-
-# warnings.formatwarning = warning_on_one_line
-
-
-# # FONCTIONS A CLASSIFIER ######################################################################################
+##############################################################################
+warnings.formatwarning = warning_on_one_line
 
 
 class target:
@@ -76,7 +57,8 @@ class target:
     Parameters
     ----------
     targname : str or int
-        Name of the target as a string, e.g. "TOI-175" or, if mission
+        Name of the target as a string, e.g. "TOI-175" or, if mission is passed
+        as the numerical identifier of the input catalog.
     mission : str, optional
         'Kepler', 'K2', or 'TESS'
 
@@ -106,6 +88,37 @@ class target:
     flux_err_kw : str
         Keyword for the column containing the flux uncertainty values
         (Default: 'flux_err')
+    KIC : str
+        For Kepler targets. Identifier in the format of the Kepler input
+        catalog, e.g. "KIC11904151"
+    kep : str
+        For Kepler targets. Identifier in the format of the Kepler mission
+        catalog, e.g. "Kepler-10"
+    KIC_num : int
+        For Kepler targets. Number of the input catalog, e.g. "11904151"
+    EPIC : str
+        For K2 targets. Identifier in the format of the Ecliptic Plane input
+        catalog, e.g. "EPIC201437844"
+    K2 : str
+        For K2 targets. Identifier in the format of the K2 mission
+        catalog, e.g. "K2-109"
+    EPIC_num : int
+        For K2 targets. Number of the input catalog, e.g. "201437844"
+    TIC : str
+        For TESS targets. Identifier in the format of the TESS input catalog,
+        e.g. "TIC307210830"
+    TOI : str
+        For TESS targets. Identifier in the format of the TESS mission catalog,
+        e.g. "TOI-175"
+    TIC_num : int
+        For TESS targets. Number of the input catalog,
+        e.g. "307210830"
+    lc_folder : str
+        Path to the folder where data is produced.
+    ts_stitch : `~astropy.timeseries.TimeSeries`
+            TimeSeries object combining data from selected lightcurves.
+    ts_clean : `~astropy.timeseries.TimeSeries`
+            TimeSeries object with trends removed.
 
     Notes
     -----
@@ -132,14 +145,14 @@ class target:
                 warnings.warn('Please add the catalog prefix and/or \
                     the mission parameter to your input.', FulmarWarning)
             else:  # Detect mission from the prefix
-                self.target_identifier(targname, None)
+                self.mission = mission_identifier(targname)
 
         self.mission = fulmar_constants.MISSION_DIC[self.mission.lower()]
 
         # Identifiers and stellar parameters
         if self.mission == 'TESS':
             # TESS identifiers
-            self.TIC, self.TOI, self.TIC_num = self.target_identifier(
+            self.TIC, self.TOI, self.TIC_num = target_identifier(
                 targname, self.mission)
 
             # Stellar parameters from TIC
@@ -147,7 +160,7 @@ class target:
                 TIC_ID=self.TIC_num)
         elif self.mission == 'Kepler':
             # Kepler identifiers
-            self.KIC, self.kep, self.KIC_num = self.target_identifier(
+            self.KIC, self.kep, self.KIC_num = target_identifier(
                 targname, self.mission)
 
             # Stellar parameters from KIC
@@ -155,7 +168,7 @@ class target:
                 KIC_ID=self.KIC_num)
         elif self.mission == 'K2':
             # K2 identifiers
-            self.EPIC, self.K2, self.EPIC_num = self.target_identifier(
+            self.EPIC, self.K2, self.EPIC_num = target_identifier(
                 targname, self.mission)
 
             # Stellar parameters from EPIC
@@ -182,92 +195,6 @@ class target:
             warnings.warn(self.mission, 'mission is not supported... yet(?)',
                           FulmarWarning)
 
-    def target_identifier(self, target, mission=None):
-        """Translate the target identifiers between different catalogs
-        such as TIC to TOI in the case of TESS or EPIC to K" for K2
-        Updates the mission parameter in case it wasn't passed by the user.
-        """
-        if target[:3].upper() == 'TIC':
-            inputCatalogID = 'TIC' + str(''.join(filter(str.isdigit, target)))
-            tic2toi = read_json_dic(
-                path.join(fulmar_constants.fulmar_dir, 'TIC2TOI.json'))
-            missionCatalogID = tic2toi[inputCatalogID]
-            ICnum = int(inputCatalogID[3:])
-            self.mission = 'TESS'
-
-        elif target[:3].upper() == 'TOI':
-            missionCatalogID = 'TOI-' + \
-                str(''.join(filter(str.isdigit, target)))
-            toi2tic = read_json_dic(
-                path.join(fulmar_constants.fulmar_dir, 'TOI2TIC.json'))
-            inputCatalogID = toi2tic[missionCatalogID]
-            ICnum = int(inputCatalogID[3:])
-            self.mission = 'TESS'
-
-        elif target[:3].upper() == 'KIC':
-            inputCatalogID = 'KIC' + str(''.join(filter(str.isdigit, target)))
-            kic2kepler = read_json_dic(
-                path.join(fulmar_constants.fulmar_dir, 'KIC2Kepler.json'))
-            missionCatalogID = kic2kepler[inputCatalogID]
-            ICnum = int(inputCatalogID[3:])
-            self.mission = 'Kepler'
-
-        elif target[:3].upper() == 'KEP':
-            missionCatalogID = 'Kepler-' + \
-                str(''.join(filter(str.isdigit, target)))
-            kep2kic = read_json_dic(
-                path.join(fulmar_constants.fulmar_dir, 'Kepler2KIC.json'))
-            inputCatalogID = kep2kic[missionCatalogID]
-            ICnum = int(inputCatalogID[3:])
-            self.mission = 'Kepler'
-
-        elif target[:4].upper() == 'EPIC':
-            inputCatalogID = 'EPIC' + str(''.join(filter(str.isdigit, target)))
-            epic2k2 = read_json_dic(
-                path.join(fulmar_constants.fulmar_dir, 'EPIC2K2.json'))
-            missionCatalogID = epic2k2[inputCatalogID]
-            ICnum = int(inputCatalogID[4:])
-            self.mission = 'K2'
-
-        elif target[:2].upper() == 'K2':
-            missionCatalogID = 'K2-' + \
-                str(''.join(filter(str.isdigit, target[2:])))
-            k22epic = read_json_dic(
-                path.join(fulmar_constants.fulmar_dir, 'K22EPIC.json'))
-            inputCatalogID = k22epic[missionCatalogID]
-            ICnum = int(inputCatalogID[4:])
-            self.mission = 'K2'
-
-        elif target.isdigit():
-            if mission == 'TESS':
-                inputCatalogID = 'TIC' + target
-                tic2toi = read_json_dic(
-                    path.join(fulmar_constants.fulmar_dir, 'TIC2TOI.json'))
-                missionCatalogID = tic2toi[inputCatalogID]
-                ICnum = int(inputCatalogID[3:])
-                warnings.warn('No prefix was passed, target is assumed to be \
-                    TIC {}'.format(ICnum), FulmarWarning)
-
-            elif mission == 'Kepler':
-                inputCatalogID = 'KIC' + target
-                kic2kepler = read_json_dic(
-                    path.join(fulmar_constants.fulmar_dir, 'KIC2Kepler.json'))
-                missionCatalogID = kic2kepler[inputCatalogID]
-                ICnum = int(inputCatalogID[3:])
-                warnings.warn('No prefix was passed, target is assumed to be \
-                    KIC {}'.format(ICnum), FulmarWarning)
-
-            elif mission == 'K2':
-                inputCatalogID = 'EPIC' + target
-                epic2k2 = read_json_dic(
-                    path.join(fulmar_constants.fulmar_dir, 'EPIC2K2.json'))
-                missionCatalogID = epic2k2[inputCatalogID]
-                ICnum = int(inputCatalogID[4:])
-                warnings.warn('No prefix was passed, target is assumed to be \
-                    EPIC {}'.format(ICnum), FulmarWarning)
-
-        return inputCatalogID, missionCatalogID, ICnum
-
     # Setters
 
     def set_lc_folder(self, inptfolder):
@@ -347,12 +274,12 @@ class target:
 
         if download is True:
             self.srch.download_all(download_dir=self.lc_folder)
-            self.lc_files = [path.as_posix() for path in Path(
+            self.lc_files = [pth.as_posix() for pth in Path(
                 self.lc_folder).rglob('*lc.fits')]
 
         sctrs = np.unique(self.srch.mission)
         self.author = np.unique(self.srch.author)
-        self.exptime = np.unique(self.srch.exptime)
+        self.exptime = self.srch.exptime
         print(sctrs)
         return self.srch
 
@@ -380,10 +307,17 @@ class target:
         lc_col = lk.LightCurveCollection([])
 
         if filelist is None:
+            print('Searching for lightcurves')
             srch = self.search_data(author=author,
                                     exptime=exptime, download=True)
+            print(len(srch), 'Lightcurves found')
+            print('Downloading Lightcurves')
             lc_col = srch.download_all()
 
+            # Add an exptime column
+            for i, lc in enumerate(lc_col):
+                lc['exptime'] = np.ones(len(lc)) * srch.exptime[i]
+
             if author is None:
                 try:
                     if len(self.author) > 1:
@@ -398,12 +332,13 @@ class target:
 
             if exptime is None:
                 try:
-                    if len(self.exptime) > 1:
+                    if len(np.unique(self.exptime)) > 1:
                         warnings.warn("You are combining data with different "
                                       "exposure times ({}), which is probably "
                                       "not what you want. If such is your "
                                       "goal, please provide an 'exptime' "
-                                      "parameter.".format(self.exptime),
+                                      "parameter.".format(
+                                          np.unique(self.exptime)),
                                       FulmarWarning)
                 except TypeError:
                     pass
@@ -415,11 +350,12 @@ class target:
                                                 colnames=colnames))
         stitched_lc = lc_col.stitch(corrector_func=lambda x: normalize_lc(x))
         self.ts_stitch = TimeSeries(stitched_lc)
+        self.ts_stitch.sort('time')
         return self.ts_stitch
 
     # Work the data
 
-    def mask_outliers(self, timeseries=None, sigma=3,
+    def mask_outliers(self, timeseries=None, sigma=5,
                       sigma_lower=None, sigma_upper=None):
         """Creates a mask to remove outliers from the lightcurve
         with special care to avoid removing transits.
@@ -445,53 +381,223 @@ class target:
 
         if timeseries is None:
             flux = self.ts_stitch[self.flux_kw]
+            exptimes = self.ts_stitch['exptime']
 
         else:
             flux = timeseries[self.flux_kw]
+            exptimes = timeseries['exptime']
 
         if len(flux) == 0:
             warnings.warn('flux is empty. Try updating the flux_kw',
                           FulmarWarning)
 
-        robustmean, robustmedian, robustrms = sigma_clipped_stats(
-            flux, sigma=sigma, maxiters=10)
+        # Creating the mask to be returned
+        clean = np.full(len(flux), False, dtype=bool)
 
-        # remove bottom outliers with a sigma clipping
-        # Indices of invalid data
-        cleanlow = np.where((flux - robustmean) / robustrms < -sigma)[0]
+        # Mask for each exptime
+        for exp in np.unique(exptimes):
+            mask_exp = np.array(exptimes == exp)
 
-        # finding 3 consecutive outliers (for transits)
-        diff = -(cleanlow - np.roll(cleanlow, -1))
-        diff2 = (cleanlow - np.roll(cleanlow, 1))
-        flux4 = flux.copy()
+            # Compute robust stats
+            robustmean, robustmedian, robustrms = sigma_clipped_stats(
+                flux[mask_exp], sigma=sigma, maxiters=10)
 
-        for i in range(0, len(cleanlow)):
+            # Remove bottom outliers with a sigma clipping
+            # Possible lower outliers
+            low_out = np.less((flux - robustmean) / robustrms, -sigma)
+            # Combine with the correct exptime mask
+            low_out_exp = np.logical_and(mask_exp, low_out)
+            # Indices of invalid data
+            cleanlow = np.where(low_out_exp)[0]
 
-            if np.logical_and(np.not_equal(diff[i], 1),
-                              np.not_equal(diff2[i],
-                                           1)):  # true outliers are set to 0
-                flux4[cleanlow[i]] = 0
+            # finding 3 consecutive outliers (for transits)
+            diff = -(cleanlow - np.roll(cleanlow, -1))
+            diff2 = (cleanlow - np.roll(cleanlow, 1))
+            flux4 = flux.copy()
 
-        # Creating the mask
-        clean = np.logical_and(np.less((flux - robustmean) / robustrms, sigma),
-                               np.not_equal(flux4, 0))
+            for i in range(0, len(cleanlow)):
 
-        # If sigma_upper or sigma_lower are passed, combines it with the
-        # Previous mask, not caring for transits
-        if sigma_upper is not None or sigma_lower is not None:
-            clip = sigma_clip(flux, sigma_upper=sigma_upper,
-                              sigma_lower=sigma_lower)
-            mask = np.logical_or(~clean, clip.mask)  # Outliers flagged as True
-            return ~mask
+                if np.logical_and(np.not_equal(diff[i], 1),
+                                  np.not_equal(diff2[i],
+                                               1)):  # Outliers are set to 0
+                    flux4[cleanlow[i]] = 0
+
+            # Creating the mask for this exptime
+            # Upper outliers as False
+            up_out = np.less((flux - robustmean) / robustrms, sigma)
+            up_out_exp = np.logical_and(mask_exp, up_out)
+
+            # Combining the masks to keep valid data
+            clean_exp = np.logical_and(up_out_exp, np.not_equal(flux4, 0))
+
+            # If sigma_upper or sigma_lower are passed, combines it with the
+            # Previous mask, not caring for transits
+            if sigma_upper is not None or sigma_lower is not None:
+                # clip = sigma_clip(flux, sigma_upper=sigma_upper,
+                #                   sigma_lower=sigma_lower)
+
+                clip_top = np.greater(
+                    (flux - robustmean) / robustrms, sigma_upper)
+                clip_low = np.less(
+                    (flux - robustmean) / robustrms, -sigma_lower)
+                clip = np.logical_or(clip_low, clip_top)  # Outliers are True
+                clip_exp = np.logical_and(
+                    ~clip, mask_exp)  # Outliers are False
+
+                # Update the mask to be returned
+                clean = np.logical_or(clean, clip_exp)
+
+            else:
+                clean = np.logical_or(clean, clean_exp)
 
         return clean
 
+    # def clean_subt_activity_flatten(
+    #         self,
+    #         timeseries=None,
+    #         sigma=3,
+    #         wl=1501,
+    #         time_window=18 * u.h,
+    #         polyorder=2,
+    #         return_trend=False,
+    #         remove_outliers=True,
+    #         break_tolerance=5,
+    #         niters=3,
+    #         mask=None):
+    #     """Removes the low frequency trend using scipy's Savitzky-Golay filter.
+    #     This method wraps `scipy.signal.savgol_filter`.
+    #     Parameters
+    #     ----------
+    #     sigma : float, optional
+    #         Number of standard deviations to use for the clipping limit.
+    #     timeseries : `~astropy.timeseries.TimeSeries`, optional
+    #         TimeSeries ot Table object containing the data to filter
+    #     wl : int, optional
+    #         Window_length
+    #         The length of the filter window (i.e. the number of coefficients).
+    #         ``window_length`` must be a positive odd integer.
+    #     time_window : '~astropy.units.Quantity' or float, optional
+    #         Time length of the filter window. Window_lenght will be set to the
+    #         closest odd integer taking exposition time into account.
+    #         Overrides wl.
+    #     polyorder : int, optional
+    #         The order of the polynomial used to fit the samples. ``polyorder``
+    #         must be less than window_length.
+    #     return_trend : bool, optional
+    #         If `True`, the method will return a tuple of two elements
+    #         (ts_clean, trend_ts) where trend_ts is the removed trend.
+    #     remove_outliers : bool
+    #         If 'True', the method uses mask_outliers to created a mask of valid
+    #         datapoints to be applied to the products before returning them.
+    #     break_tolerance : int, optional
+    #         If there are large gaps in time, flatten will split the flux into
+    #         several sub-lightcurves and apply `savgol_filter` to each
+    #         individually. A gap is defined as a period in time larger than
+    #         `break_tolerance` times the median gap.  To disable this feature,
+    #         set `break_tolerance` to None.
+    #     niters : int, optional
+    #         Number of iterations to iteratively sigma clip and flatten. If more
+    #         than one, will perform the flatten several times,
+    #         removing outliers each time.
+
+    #     mask : boolean array with length of time, optional
+    #         Boolean array to mask data with before flattening. Flux values
+    #         where mask is True will not be used to flatten the data. An
+    #         interpolated result will be provided for these points. Use this
+    #         mask to remove data you want to preserve, e.g. transits.
+
+    #     Returns
+    #     -------
+    #     ts_clean : `~astropy.timeseries.TimeSeries`
+    #         New `TimeSeries` object with long-term trends removed.
+    #     If ``return_trend`` is set to ``True``, this method will also return:
+    #     trend_ts : `~astropy.timeseries.TimeSeries`
+    #         New `TimeSeries` object containing the trend that was removed from
+    #         the flux.
+    #     """
+    #     if timeseries is None:
+    #         self.ts_clean = self.ts_stitch.copy()
+
+    #     else:
+    #         self.ts_clean = timeseries.copy()
+
+    #     flux = self.ts_clean[self.flux_kw]
+    #     lc = lk.LightCurve(self.ts_clean)
+
+    #     if mask is None:
+    #         mask = np.full_like(self.ts_clean.time.value, False, dtype=bool)
+
+    #     # Inital robust stats
+    #     robustmean1, robustmedian1, robustrms1 = sigma_clipped_stats(
+    #         flux, sigma=sigma, maxiters=10, mask=mask)
+
+    #     if time_window is not None:
+    #         if isinstance(time_window, float):
+    #             time_window = time_window * u.d
+    #         # observation time interval
+    #         dt = (self.ts_clean.time[1] - self.ts_clean.time[0]).to(
+    #             time_window.unit)
+    #         nobs = round((time_window / dt).value)
+    #         # make sure the number is odd
+    #         wl = nobs + 1 - (nobs % 2)
+
+    #     # prefiltering
+    #     # lc = lk.LightCurve(time=self.ts_stitch.time.value,
+    #     #                    flux=self.ts_stitch[self.flux_kw + '_norm'].value,
+    #     #                    flux_err=self.ts_stitch[self.flux_err_kw].value)
+
+    #     clc = lc.flatten(window_length=wl,
+    #                      polyorder=polyorder,
+    #                      break_tolerance=break_tolerance,
+    #                      sigma=sigma,
+    #                      mask=mask)
+
+    #     flux_filtered1 = clc.flux
+
+    #     # sigma clipping for big transits
+    #     clip = sigma_clip(flux_filtered1, sigma)
+
+    #     finflat = lc.flatten(window_length=wl, polyorder=polyorder,
+    #                          break_tolerance=break_tolerance, sigma=sigma,
+    #                          return_trend=True, mask=clip.mask)
+
+    #     clc1 = finflat[0]
+    #     trend_ts = TimeSeries(finflat[1])
+
+    #     flux_filtered = clc1.flux
+
+    #     # Final robust stats
+    #     robustmean, robustmedian, robustrms = sigma_clipped_stats(
+    #         flux_filtered, sigma=sigma, maxiters=10, mask=mask)
+
+    #     # Warn the user if std_dev is bigger than the initial.
+    #     if robustrms > robustrms1:
+    #         warnings.warn('Standard deviaton of the flux did not decrease \
+    #             after filtering (before : {}, after : {}). Try using a mask\
+    #             for the transits' .format(robustrms1, robustrms),
+    #                       FulmarWarning)
+
+    #     self.ts_clean[self.flux_kw] = flux_filtered
+
+    #     if remove_outliers is True:
+    #         out_mask = self.mask_outliers(timeseries=self.ts_clean,
+    #                                       sigma=sigma)
+    #         self.ts_clean = self.ts_clean[out_mask]
+
+    #         trend_ts = trend_ts[out_mask]
+
+    #     if return_trend is True:
+    #         trend_ts = trend_ts[out_mask]
+
+    #         return self.ts_clean, trend_ts
+
+    #     return self.ts_clean
     def clean_subt_activity_flatten(
             self,
             timeseries=None,
             sigma=3,
             wl=1501,
-            time_window=None,
+            time_window=18 * u.h,
             polyorder=2,
             return_trend=False,
             remove_outliers=True,
@@ -506,14 +612,14 @@ class target:
             Number of standard deviations to use for the clipping limit.
         timeseries : `~astropy.timeseries.TimeSeries`, optional
             TimeSeries ot Table object containing the data to filter
-        wl : int
+        wl : int, optional
             Window_length
             The length of the filter window (i.e. the number of coefficients).
             ``window_length`` must be a positive odd integer.
         time_window : '~astropy.units.Quantity' or float, optional
             Time length of the filter window. Window_lenght will be set to the
             closest odd integer taking exposition time into account.
-            Overrules wl.
+            Overrides wl.
         polyorder : int, optional
             The order of the polynomial used to fit the samples. ``polyorder``
             must be less than window_length.
@@ -555,63 +661,97 @@ class target:
         else:
             self.ts_clean = timeseries.copy()
 
-        flux = self.ts_stitch[self.flux_kw]
-        lc = lk.LightCurve(self.ts_stitch)
-
-        if mask is None:
-            mask = np.full_like(self.ts_clean.time.value, False, dtype=bool)
-
-        # Inital robust stats
-        robustmean1, robustmedian1, robustrms1 = sigma_clipped_stats(
-            flux, sigma=sigma, maxiters=10, mask=mask)
+        exptimes = self.ts_clean['exptime']
 
         if time_window is not None:
             if isinstance(time_window, float):
                 time_window = time_window * u.d
-            # observation time interval
-            dt = (self.ts_clean.time[1] - self.ts_clean.time[0]).to(
-                time_window.unit)
-            nobs = round((time_window / dt).value)
-            # make sure the number is odd
-            wl = nobs + 1 - (nobs % 2)
+        else:
+            if len(np.unique(exptimes)) > 1:
+                warnings.warn('You are using a fixed window lenght'
+                              'wl = {} but you have '.format(wl) + 'multiple '
+                              'exptimes ({}).'.format(exptimes) + 'This will '
+                              'result in a variable time_window for the '
+                              'SG filter. Which you do not want.'
+                              'Try setting a time_window instead',
+                              FulmarWarning)
+
+        # flux = self.ts_clean[self.flux_kw]
+        # lc = lk.LightCurve(self.ts_clean)
+
+        if mask is None:
+            mask = np.full(len(self.ts_clean), False, dtype=bool)
+
+        clean_col = lk.LightCurveCollection([])
+        trend_col = lk.LightCurveCollection([])
+
+        for exp in np.unique(exptimes):
+            mask_exp = np.array(exptimes == exp)
+            flux = self.ts_clean[self.flux_kw][mask_exp]
+
+            # Compute robust stats
+            robustmean1, robustmedian1, robustrms1 = sigma_clipped_stats(
+                flux, sigma=sigma, maxiters=10, mask=mask[mask_exp])
+
+            lc = lk.LightCurve(self.ts_clean[mask_exp])
+
+        # # Inital robust stats
+        # robustmean1, robustmedian1, robustrms1 = sigma_clipped_stats(
+        #     flux, sigma=sigma, maxiters=10, mask=mask)
+
+        # Define the correct window_lenght
+            if time_window is not None:
+                # observation time interval
+                dt = exp.to(time_window.unit)
+                nobs = round((time_window / dt).value)
+                # make sure the number is odd
+                wl = nobs + 1 - (nobs % 2)
 
         # prefiltering
         # lc = lk.LightCurve(time=self.ts_stitch.time.value,
         #                    flux=self.ts_stitch[self.flux_kw + '_norm'].value,
         #                    flux_err=self.ts_stitch[self.flux_err_kw].value)
 
-        clc = lc.flatten(window_length=wl,
-                         polyorder=polyorder,
-                         break_tolerance=break_tolerance,
-                         sigma=sigma,
-                         mask=mask)
+            clc = lc.flatten(window_length=wl,
+                             polyorder=polyorder,
+                             break_tolerance=break_tolerance,
+                             sigma=sigma,
+                             mask=mask[mask_exp])
 
-        flux_filtered1 = clc.flux
+            flux_filtered1 = clc.flux
 
         # sigma clipping for big transits
-        clip = sigma_clip(flux_filtered1, sigma)
+            clip = sigma_clip(flux_filtered1, sigma)
 
-        finflat = lc.flatten(window_length=wl, polyorder=polyorder,
-                             break_tolerance=break_tolerance, sigma=sigma,
-                             return_trend=True, mask=clip.mask)
+            finflat = lc.flatten(window_length=wl, polyorder=polyorder,
+                                 break_tolerance=break_tolerance, sigma=sigma,
+                                 return_trend=True, mask=clip.mask)
 
-        clc1 = finflat[0]
-        trend_ts = TimeSeries(finflat[1])
+            # clc1 = finflat[0]
+            # trend_ts = TimeSeries(finflat[1])
 
-        flux_filtered = clc1.flux
+            clean_col.append(finflat[0])
+            trend_col.append(finflat[1])
 
-        # Final robust stats
-        robustmean, robustmedian, robustrms = sigma_clipped_stats(
-            flux_filtered, sigma=sigma, maxiters=10, mask=mask)
+            # Final robust stats
+            robustmean, robustmedian, robustrms = sigma_clipped_stats(
+                finflat[0].flux, sigma=sigma, maxiters=10, mask=mask[mask_exp])
 
-        # Warn the user if std_dev is bigger than the initial.
-        if robustrms > robustrms1:
-            warnings.warn('Standard deviaton of the flux did not decrease \
-                after filtering (before : {}, after : {}). Try using a mask\
-                for the transits' .format(robustrms1, robustrms),
-                          FulmarWarning)
+            # Warn the user if std_dev is bigger than the initial.
+            if robustrms > robustrms1:
+                warnings.warn('Standard deviaton of the flux did not decrease '
+                              'after filtering (before : {}, after : {}). Try '
+                              'using a mask for the transits' .format(
+                                  robustrms1, robustrms),
+                              FulmarWarning)
+
+        cleaned_lc = clean_col.stitch()
+        trend_ts = TimeSeries(trend_col.stitch())
+
+        flux_filtered = cleaned_lc.flux
 
         self.ts_clean[self.flux_kw] = flux_filtered
+        self.ts_clean.sort('time')
 
         if remove_outliers is True:
             out_mask = self.mask_outliers(timeseries=self.ts_clean,
@@ -873,109 +1013,6 @@ class target:
         plt.show()
         plt.close()
 
-    # def tls_periodogram(
-    #         self,
-    #         timeseries=None,
-    #         cleaned=True,
-    #         period_min=None,
-    #         period_max=None,
-    #         n_transits_min=None,
-    #         transit_depth_min=1e-5,
-    #         mask=None):
-    #     """Computes the tls periodogram of the selected lightcurve
-    #     Parameters
-    #     ----------
-    #     timeseries : `~astropy.timeseries.TimeSeries`, optional
-    #         TimeSeries ot Table object containing the data to filter
-    #     cleaned : bool, optional
-    #         Whether the periodogram should be conducted on the cleaned or the
-    #         stitched timeseries (default: True)
-    #     period_min : float
-    #         Minimum trial period (in units of days). If none is given,
-    #         the limit is derived from the Roche limit
-    #     period_max : float
-    #         Maximum trial period (in units of days) (default: Half the duration
-    #          of the time series)
-    #     n_transits_min : int, optional
-    #         Minimum number of transits required. Overrules period_max.
-    #         (default=2)
-    #     mask : boolean array with length of time
-    #         Boolean array to mask data, typically transits. Data where mask is
-    #         "True" will not be taken into account for the periodogram.
-    #     Returns
-    #     -------
-    #     tls_results : transitleastsquaresresults
-    #     """
-
-    #     if timeseries is None:
-
-    #         if cleaned is True:
-    #             t = self.ts_clean.time.value
-    #             y = np.array(
-    #                 self.ts_clean[self.flux_kw], dtype=np.float64)
-    #             y_err = np.array(
-    #                 self.ts_clean[self.flux_err_kw], dtype=np.float64)
-    #         else:
-    #             t = self.ts_stitch.time.value
-    #             y = np.array(
-    #                 self.ts_stitch[self.flux_kw], dtype=np.float64)
-    #             y_err = np.array(
-    #                 self.ts_stitch[self.flux_err_kw], dtype=np.float64)
-    #     else:
-    #         t = timeseries.time.value
-    #         y = np.array(timeseries[self.flux_kw], dtype=np.float64)
-    #         y_err = np.array(timeseries[self.flux_err_kw], dtype=np.float64)
-
-    #     # Accounts for possible mask
-    #     if mask is not None:  # intransit = True
-    #         t = t[~mask]
-    #         y = y[~mask]
-    #         y_err = y_err[~mask]
-
-    #     # Initialize the tls model
-
-    #     # Case 1: y_err only contains NaNs (common with EVEREST)
-    #     if len(cleaned_array(t, y, y_err)[0]) == 0:
-    #         t, y = cleaned_array(t, y)
-    #         model = transitleastsquares(t, y)
-    #     # Case 2: y_err contains (at least some) valid data
-    #     else:
-    #         t, y, yerr = cleaned_array(t, y, y_err)
-    #         model = transitleastsquares(t, y, y_err)
-
-    #     if n_transits_min is None:
-    #         if period_max is not None:
-    #             n_transits_min = int((max(t) - min(t)) / period_max)
-    #         else:
-    #             n_transits_min = 2
-
-    #     if period_max is None:
-    #         period_max = (max(t) - min(t)) / n_transits_min
-
-    #     if period_min is None:
-    #         # Compute the periodogram
-    #         self.tls_results = model.power(
-    #             u=self.ab, R_star=self.R_star,
-    #             R_star_min=self.R_star - self.R_star_min,
-    #             R_star_max=self.R_star + self.R_star_max,
-    #             M_star=self.M_star, M_star_min=self.M_star - self.M_star_min,
-    #             M_star_max=self.M_star + self.M_star_max,
-    #             period_max=period_max, n_transits_min=n_transits_min,
-    #             transit_depth_min=transit_depth_min)
-    #     else:
-    #         # Compute the periodogram
-    #         self.tls_results = model.power(
-    #             u=self.ab, R_star=self.R_star,
-    #             R_star_min=self.R_star - self.R_star_min,
-    #             R_star_max=self.R_star + self.R_star_max,
-    #             M_star=self.M_star, M_star_min=self.M_star - self.M_star_min,
-    #             M_star_max=self.M_star + self.M_star_max,
-    #             period_min=period_min, period_max=period_max,
-    #             n_transits_min=n_transits_min,
-    #             transit_depth_min=transit_depth_min)
-
-    #     return self.tls_results
-
     def tls_periodogram(
             self,
             timeseries=None,
@@ -1065,5 +1102,3 @@ class target:
 # # Create a transit/planet object?
 # class transit:
 #     "A transit object with its associated parameters"
-
-
